import java.awt.image.*;
import java.util.Random;

class AutomateCellulaire implements Runnable
{
    private Thread threadDeMiseAJour;
    private byte[][] t0,t1;
    private int nbc,nbl;
    private boolean misAJour = true;
    private BufferedImage renduVisuel;
    private WritableRaster tamponRenduVisuel;

    AutomateCellulaire(int nbc,int nbl,BufferedImage renduVisuel)
    {
	Random alatoire = new Random();
        this.nbc = nbc;
        this.nbl = nbl;
	this.renduVisuel = renduVisuel;
	tamponRenduVisuel = renduVisuel.getRaster();
        // instanciation des deux tableaux de cellules :
        t0 = new byte[nbl][nbc];
        t1 = new byte[nbl][nbc];

        // initialisation du tableau t1 de cellules :
        for(int i=0 ; i<nbl ; i++)
            for(int j=0 ; j<nbc ; j++) {
                if ((i==0) || (j==0) || (i==nbl-1) || (j==nbc-1))
		    // les cellules du bord ne peuvent pas bruler :
                    t1[i][j] = 0;
                else if (i == 1)
		    // les cellules du bord suprieur sont enflammes :
                    t1[i][j] = 13;
                else 
		    // les autres cellules sont susceptibles de s'enflammer :
                    t1[i][j] = (byte)
                        ((alatoire.nextInt(100) < 60) ? 14 : 0);
		tamponRenduVisuel.setPixel(j,i,colorier(i,j));
            }
	threadDeMiseAJour = new Thread(this);
	threadDeMiseAJour.setPriority(Thread.MIN_PRIORITY);
	threadDeMiseAJour.start();
    }
    
    synchronized public void setMisAJour(boolean valeur) 
    {
	misAJour = valeur;
    }

    synchronized public boolean getMisAJour() 
    {
	return misAJour;
    }

    void transitionGlobale() 
    {
        // on commence par permuter les deux tableaux t0 et t1 :
        byte[][] tmp = t0;
	t0 = t1;
	t1 = tmp;
        // puis on applique la fonction de transition locale aux cellules internes :
        for(int i=1 ; i<nbl-1 ; i++)
            for(int j=1 ; j<nbc-1 ; j++) {
                t1[i][j] = t0[i][j];
                // la cellule courante brle si et seulement elle se consume
                // dj ou si elle n'est pas ininflammable et si l'un de ses
                // voisins au moins (au sens du voisinage de von Neumann)
                // brle :
                if ((t0[i][j] >= 1 && t0[i][j] <= 13) ||
                    (t0[i][j] == 14 && 
                     (t0[i-1][j] == 13 || t0[i+1][j] == 13 || 
                      t0[i][j-1] == 13 || t0[i][j+1] == 13)
                     ))
                    t1[i][j] -= 1;
		tamponRenduVisuel.setPixel(j,i,colorier(i,j));
            }
    }
    
    int[] colorier(int i,int j)
    {
	// cette mthode se charge d'associer un code de couleur  la cellule
	// courante en fonction de la valeur de son tat :
	int rouge = 0 , vert = 0 , bleu = 0;
	switch (t1[i][j]) {
	case 0:
	    bleu = 255;
	    break;
	case 14:
	    vert = 255;
	    break;
	default:
	    rouge = (255 * (13-t1[i][j])) / 13;
	}
	return new int[] {rouge,vert,bleu};
    }

    public void run()
    {
	while (true)
	    if (! getMisAJour()) {
		transitionGlobale();
		setMisAJour(true);
	    }
    }
}
