import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/* Lors de la dfinition de la classe DisqueRectangleTriangle, nous n'avons
 * pas choisi d'implmenter l'interface WindowListener et ses 7 mthodes
 * windowActivated, windowClosed, windowClosing, windowDeactivated,
 * windowDeiconified, windowIconified et windowOpened, pour simplifier
 * l'expos du code qui suit. Par contre, comme nous devons grer le focus
 * lors de l'activation de la fentre, nous avons redfini la mthode
 * windowActivated hrite de la classe abstraite WindowAdapter : */

class DisqueRectangleTriangle extends WindowAdapter
{
    JFrame laFentre;
    /* Dfinition de la classe interne ArdoiseGraphique. Ici, contrairement au
     * cas de la classe DisqueRectangleTriangle, comme cette classe ne peut
     * pas hriter de la classe abstraite KeyAdapter (car il y aurait hritage
     * multiple), nous faisons en sorte qu'elle implmente l'interface
     * KeyListener : */
    class ArdoiseGraphique extends JPanel implements KeyListener
    {
	char choix = 'd';

	ArdoiseGraphique()
	{
	    // on redimensionne le composant courant :
	    setPreferredSize(new Dimension(240,150));
	    // on lui associe un gestionnaire de frappe des touches qui est
	    // oprationnel lorsque le focus est dans l'instance
	    // d'ArdoiseGraphique :
	    addKeyListener(this);
	}

	public void paintComponent(Graphics g) {
	    // appel de la mthode paintComponent de la classe mre pour
	    // r-initialiser le contexte graphique :
	    super.paintComponent(g);
	    // le trac est fonction de la valeur du "choix" :
	    switch (choix) {
	    case 'd':
		g.fillOval(70,25,100,100);
		break;
	    case 'r':
		g.fillRect(40,25,160,100);
		break;
	    case 't':
		g.fillPolygon(new int[]{120,80,160}, new int[]{25,125,125},3);
		break;
	    }
	}
	// comme la classe ArdoiseGraphique implmente l'interface
	// KeyListener, il faut imprativement dfinir les mthodes
	// keyPressed, keyReleased et keyTyped :
	public void keyPressed(KeyEvent vnement) {}
	public void keyReleased(KeyEvent vnement) {}
	public void keyTyped(KeyEvent vnement) 
	{
	    // on rcupre la valeur de la touche qui a t enfonce et on
	    // applique un traitement circonstanci en fonction de celle-ci :
	    switch (choix = vnement.getKeyChar()) {
	    case 'd':
	    case 'r':
	    case 't':
		// on active la mthode repaint qui se charge de l'appel  la
		// mthode paintComponent dfinie ci-avant :
		repaint();
		break;
	    case 'q':
		// dans ce cas, on termine l'application :
		laFentre.setVisible(false);
		laFentre.dispose();
		System.exit(0);
	    }
	}
    }
    // constructeur de la classe englobante :
    DisqueRectangleTriangle()
    {
	// on commence par crer la fentre :
	laFentre = new JFrame("Figures gomtriques");
	// on lui attache un gestionnaire d'vnements :
	laFentre.addWindowListener(this);
	// on instancie une "ArdoiseGraphique" que l'on attache  "laFentre" :
	laFentre.setContentPane(new ArdoiseGraphique());
	// enfin, on procde  l'affichage de l'ensemble :
        laFentre.pack();
        laFentre.setVisible(true);
    }

    public void windowActivated(WindowEvent vnement)
    {                 
	laFentre.getContentPane().requestFocus();    
    }

    public static void main (String[] arguments) throws Exception
    {
	// on lance l'application en instanciant la classe englobante
	// DisqueRectangleTriangle :
	new DisqueRectangleTriangle();
    }
}
