import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/* la classe MiniCalculatrice englobe une premire classe interne
 * "BarreDeMenus" (charge d'implmenter la barre de menus) et une
 * seconde "PavNumrique" (charge d'implmenter le pav numrique et
 * la zone d'affichage de la calculatrice) : */
class MiniCalculatrice implements ActionListener
{
    JFrame laFentre;
    JLabel zoneDAffichage;
    long oprandeGauche=0,oprandeDroit=0;
    char clicPrcdent=' ';
    /* la classe interne "BarreDeMenus", qui drive de la classe
     * JMenuBar, englobe elle-mme une classe interne "ChoixDuMenu"
     * qui permet d'implmenter les trois choix de l'unique menu
     * droulant de notre application : */
    class BarreDeMenus extends JMenuBar
    {
	class ChoixDuMenu extends JMenuItem
	{
	    ChoixDuMenu(MiniCalculatrice xcalc,String tiquette,char raccourciClavier)
	    {
		super(tiquette,raccourciClavier);
		setActionCommand(String.valueOf(raccourciClavier));
		addActionListener(xcalc);
	    }
	}

	BarreDeMenus(MiniCalculatrice xcalc)
	{
	    JMenu uniqueMenu = new JMenu("Application");
	    uniqueMenu.add(new ChoixDuMenu(xcalc,"Pour information",'i'));
	    uniqueMenu.add(new ChoixDuMenu(xcalc,"R-initialiser",'r'));
	    uniqueMenu.add(new ChoixDuMenu(xcalc,"Quitter",'q'));
	    add(uniqueMenu);
	}
    }
    /* la classe interne "PavNumrique", qui drive de la classe
     * JPanel, englobe elle-mme une classe interne "Bouton", qui
     * drive de JButton et est charge de l'implmentation de chacun
     * des boutons de la calculatrice. */
    class PavNumrique extends JPanel
    {
	GridBagLayout grilleDePlacement;
	GridBagConstraints contraintesDePlacement;
	Font laFonte = new Font("Monospaced",Font.BOLD,36);

	class Bouton extends JButton
	{
	    Bouton(MiniCalculatrice xcalc,PavNumrique leConteneur,
		   String tiquette,int contrainteEnRange)
	    {
		super(tiquette);
		setFont(laFonte);
		contraintesDePlacement.gridwidth = contrainteEnRange;
		grilleDePlacement.setConstraints(this,contraintesDePlacement);
		leConteneur.add(this);
		setActionCommand(tiquette);
		addActionListener(xcalc);
	    }
	}

	PavNumrique(MiniCalculatrice xcalc)
	{
	    /* pour mettre en place le pav numrique, nous avons
	     * utilis un gestionnaire de placement de type
	     * GridBagLayout. Ce gestionnaire permet de disposer les
	     * boutons de la calculatrice et sa zone d'affichage sur
	     * une grille de 5 ranges de 3 colonnes (disposition
	     * habituelle d'une mini-calculatrice). Ce placement
	     * s'effectue en fonction d'une instance de
	     * GridBagConstraints, celle-ci est charge d'exprimer les
	     * contraintes de positionnement de chaque composant au
	     * sein de la grille. */
	    grilleDePlacement = new GridBagLayout();
	    contraintesDePlacement = new GridBagConstraints();
	    // le gestionnaire de placement est rattach au composant
	    // courant driv de JPanel :
	    setLayout(grilleDePlacement);
	    /* quand le composant (le bouton ou la zone d'affichage)
	     * est plus petit que l'emplacement de la grille qui lui
	     * est allou, celui-ci est "requalibr" dans les deux
	     * dimensions pour occuper toute la place disponible : */
	    contraintesDePlacement.fill = GridBagConstraints.BOTH;
	    // instanciation de la zone d'affichage :
	    zoneDAffichage = new JLabel("0",JLabel.RIGHT);
	    zoneDAffichage.setFont(laFonte);
	    // la constante "GridBagConstraints.REMAINDER" permet de
	    // spcifier que le composant courant est le dernier de sa
	    // range (si elle est affecte  l'attribut "gridwidth"
	    // du moins) :
	    contraintesDePlacement.gridwidth = GridBagConstraints.REMAINDER;
	    grilleDePlacement.setConstraints(zoneDAffichage,contraintesDePlacement);
	    add(zoneDAffichage);
	    // instanciation des boutons :
	    new Bouton(xcalc,this,"7",1);
	    new Bouton(xcalc,this,"8",1);
	    new Bouton(xcalc,this,"9",GridBagConstraints.REMAINDER);

	    new Bouton(xcalc,this,"4",1);
	    new Bouton(xcalc,this,"5",1);
	    new Bouton(xcalc,this,"6",GridBagConstraints.REMAINDER);

	    new Bouton(xcalc,this,"1",1);
	    new Bouton(xcalc,this,"2",1);
	    new Bouton(xcalc,this,"3",GridBagConstraints.REMAINDER);

	    new Bouton(xcalc,this,"0",1);
	    new Bouton(xcalc,this,"=",1);
	    new Bouton(xcalc,this,"+",GridBagConstraints.REMAINDER);
	}
    }
    // constructeur de la classe englobante :
    MiniCalculatrice()
    {
	// on commence par crer la fentre et lui dfinir un
	// comportement  la fermeture :
	laFentre = new JFrame("XCalc");
	laFentre.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	// on instancie une barre de menus que l'on rattache 
	// "laFentre" :
	laFentre.setJMenuBar(new BarreDeMenus(this));
	// ainsi qu'un "PavNumrique" :
	laFentre.setContentPane(new PavNumrique(this));
	// enfin, on procde  l'affichage de l'ensemble :
        laFentre.pack();
        laFentre.setVisible(true);
    }
    // mthode de gestion des vnements survenus au cours de
    // l'excution de la classe MiniCalculatrice :
    public void actionPerformed(ActionEvent vnement)
    {
	char rsultatDuClic = vnement.getActionCommand().charAt(0);

	switch (rsultatDuClic) {
	case 'i':
            // cration de la bote de dialogue modale d'information :
            JOptionPane leContenu = 
		new JOptionPane("Il s'agit d'une Mini-calculatrice ;-)",
				JOptionPane.INFORMATION_MESSAGE,
				JOptionPane.DEFAULT_OPTION);
            JDialog boteDeDialogue = 
		leContenu.createDialog(laFentre,"Pour information...");
            boteDeDialogue.setVisible(true);
	    break;
	case 'r':
	    // r-initialisation de la calculatrice :
	    zoneDAffichage.setText("0");
	    oprandeGauche = 0;
	    oprandeDroit = 0;
	    clicPrcdent = ' ';
	    break;
	case 'q':
	    // fin de l'application :
	    laFentre.setVisible(false);
	    laFentre.dispose();
	    System.exit(0);
	    break;
	case '+':
	    if ((clicPrcdent != '=') && (clicPrcdent != '+')) {
		oprandeDroit = Long.parseLong(zoneDAffichage.getText());
		oprandeGauche += oprandeDroit;
		zoneDAffichage.setText(String.valueOf(oprandeGauche));
	    }
	    break;
	case '=':
	    if (clicPrcdent == '+') {
		long tmp = oprandeGauche;
		oprandeGauche += oprandeDroit;
		oprandeDroit = tmp;
	    }
	    else if (clicPrcdent == '=')
		oprandeGauche += oprandeDroit;
	    else {
		oprandeDroit = Long.parseLong(zoneDAffichage.getText());
		oprandeGauche += oprandeDroit;
	    } 
	    zoneDAffichage.setText(String.valueOf(oprandeGauche));
	    break;
	default:
	    String affichage;
	    if ((clicPrcdent == '=') || (clicPrcdent == '+'))
		zoneDAffichage.setText("" + rsultatDuClic);
	    else {
		affichage = zoneDAffichage.getText();
		if (affichage.equals("0"))
		    zoneDAffichage.setText("" + rsultatDuClic);
		else if (affichage.length() < 8)
		    zoneDAffichage.setText(affichage + rsultatDuClic);
	    }
	}
	clicPrcdent = rsultatDuClic;
    }

    public static void main (String[] arguments) throws Exception
    {
	// on lance l'application en instanciant la classe englobante
	// MiniCalculatrice :
	new MiniCalculatrice();
    }
}
