import java.io.*;

class BibliothequeEnArbreBinaire implements Serializable
{
    String auteurs,titre;
    // les deux pointeurs vers les livres "fils" dans l'arbre binaire
    // de recherche :
    BibliothequeEnArbreBinaire filsGauche,filsDroit;

    // les constructeurs :
    BibliothequeEnArbreBinaire()
    {
	auteurs = null;
	titre = null;
	filsGauche = null;
	filsDroit = null;
    }

    BibliothequeEnArbreBinaire(String auteurs,String titre)
    {
	this(); // appel au constructeur par dfaut
	this.auteurs = auteurs;
	this.titre = titre;
    }

    void insrer(String auteurs,String titre)
    {
	if (this.auteurs == null) {
	    // l'arbre est vide, on l'initialise avec le premier titre :
	    this.auteurs = auteurs;
	    this.titre = titre;
	}
	else if (this.auteurs.compareTo(auteurs) >= 0) {
	    // comme l'auteur  insrer est plac avant l'auteur
	    // courant dans la relation d'ordre lexicographique
	    // applique aux noms d'auteurs, on descend dans le
	    // sous-arbre gauche :
	    if (filsGauche == null)
		// si l'on est sur une feuille on procde  l'insertion :
		filsGauche = new BibliothequeEnArbreBinaire(auteurs,titre);
	    else 
		filsGauche.insrer(auteurs,titre);
	}
	else {
	    // dans ce cas, l'auteur  insrer est plac aprs
	    // l'auteur courant dans la relation d'ordre
	    // lexicographique applique aux noms d'auteurs, on
	    // descend donc dans le sous-arbre droit :
	    if (filsDroit == null)
		// si l'on est sur une feuille on procde  l'insertion :
		filsDroit = new BibliothequeEnArbreBinaire(auteurs,titre);
	    else 
		filsDroit.insrer(auteurs,titre);	    
	}
    }

    void afficher()
    {
	// pour afficher le contenu de l'arbre binaire de recherche en
	// respectant l'ordre lexxicographique, on le parcourt de
	// manire rcursive en commenant par descendre dans la
	// portion gauche de l'arbre, puis en continuant par le
	// contenu du noeud lui-mme, avant de terminer par la portion
	// droite de l'arbre :
	if (filsGauche != null) filsGauche.afficher();
	System.out.println("AUTEUR(S) : " + auteurs + " -- TITRE : " + titre);
	if (filsDroit != null) filsDroit.afficher();
    }

    void stocker(String nomDeFichier) throws IOException
    {
	// cette mthode met en oeuvre le mcanisme de srialisation :
	ObjectOutputStream sortie = 
	    new ObjectOutputStream(new FileOutputStream(nomDeFichier));
	sortie.writeObject(this);
	sortie.flush(); // on force l'criture effective sur disque
	sortie.close();
    }

    static BibliothequeEnArbreBinaire charger(String nomDeFichier)
	throws IOException,ClassNotFoundException
    {
	// cette mthode met en oeuvre le mcanisme de dsrialisation :
	BibliothequeEnArbreBinaire tmp;
	ObjectInputStream entre = 
	    new ObjectInputStream(new FileInputStream(nomDeFichier));
	tmp = (BibliothequeEnArbreBinaire) entre.readObject();
	entre.close();
	return tmp;
    }

    void enrichirLaBibliothque() throws IOException
    {
	// dfinissons le tampon de lecture de l'entre standard :
        BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in));
	boolean continuer = true;
	String auteursTmp,titreTmp;

	do {
	    System.out.print("Nom(s) d'auteur(s) (ou retour chariot pour terminer) : ");
	    auteursTmp = stdin.readLine();
	    System.out.print("Titre (ou retour chariot pour terminer) : ");
	    titreTmp = stdin.readLine();
	    // tant que les chanes de caractres correspondant 
	    // auteursTmp et titreTmp ne sont pas vides, on continue :
	    if (! auteursTmp.equalsIgnoreCase("") && ! titreTmp.equalsIgnoreCase(""))
		insrer(auteursTmp,titreTmp);
	    else continuer = false;
        } while (continuer);
    }

    public static void main (String[] arguments)
	throws IOException,ClassNotFoundException
    {
	BibliothequeEnArbreBinaire maBNF;
	// le premier et unique argument de la ligne de commande
	// correspond au nom du fichier de la bibliothque  enrichir :
	String nomDeFichier = arguments[0];
	File leFichier = new File(nomDeFichier);

	if (leFichier.exists())
	    // le fichier existe, on procde  la dsrialisation :
	    maBNF = charger(nomDeFichier);
	else
	    maBNF = new BibliothequeEnArbreBinaire();

	// on enrichit la bibliothque :
	maBNF.enrichirLaBibliothque();

	// on stocke l'tat de la bibliothque aprs affichage :
	maBNF.afficher();
	maBNF.stocker(nomDeFichier);
    }
}
