import java.io.*;

public class Wc
{
    static long nombreL = 0;
    static long nombreW = 0;
    static long nombreC = 0;
    static boolean compterL = true;
    static boolean compterW = true;
    static boolean compterC = true;

    public static void usage () 
    {
	final String USAGE = 
	    "Usage : java Wc [-lwc] (nom de fichier(s) | -)\n"
	    + "\t-l : affiche le nombre total de lignes\n"
	    + "\t-w : affiche le nombre total de mots\n"
	    + "\t-c : affiche le nombre total de carateres\n"
	    + "\t(Sans nom de fichier ou si '-' n'est pas specifie,\n"
	    + "\tc'est l'entree standard qui est lue par defaut !)\n";
	System.err.println(USAGE);
	System.exit(1);
    }

    protected static void wcAffichage (long L,long W,long C,String nomDeFichier)
    {
	System.out.println((compterL ? "\t" + L : "") 
			   + (compterW ? "\t" + W : "") 
			   + (compterC ? "\t" + C : "")
			   + ((nomDeFichier != null) ? " " + nomDeFichier : " total")
			   );
    }
    
    protected static void wcAuxiliaire (Reader fr,String nomDeFichier) throws IOException
    {
	long nombreAuxiliaireL = 0;
	long nombreAuxiliaireW = 0;
	long nombreAuxiliaireC = 0;

	// calcul du nombre de caractres dans le flot d'entre
	if (nomDeFichier == null) {
	    // lecture de l'entre standard
	    nombreAuxiliaireC = System.in.available();
	}
	else {
	    // lecture du fichier de donnes (si possible)
	    File f = new File(nomDeFichier);
	    if (f.canRead())
		nombreAuxiliaireC = f.length();
	}
	// calcul du nombre de lignes et de mots dans le flot d'entre
	// par le biais d'un analyseur lexical :
	StreamTokenizer analyseurSimple = new StreamTokenizer(fr);
	// remise  zro des tables d'analyse lexicale :
	// tous les caractres sont identiquement traits
	analyseurSimple.resetSyntax(); 
	// tous les caractres de 0  255 sont traits comme des parties de mots
	analyseurSimple.wordChars(0,255);
	// 5 types de dlimiteurs d'units lexicales 
  	analyseurSimple.whitespaceChars('\b','\b'); 
 	analyseurSimple.whitespaceChars('\r','\r');
  	analyseurSimple.whitespaceChars('\t','\t');
  	analyseurSimple.whitespaceChars('\n','\n');
	analyseurSimple.whitespaceChars(' ',' ');
	// il n'y a pas de conversion automatique en minuscule
	// analyseurSimple.lowerCaseMode(false);
	while (analyseurSimple.nextToken() != StreamTokenizer.TT_EOF)
	    nombreAuxiliaireW++;
	nombreAuxiliaireL = analyseurSimple.lineno() - 1;
	
	// incrmentation des compteurs gnraux :
	nombreL += nombreAuxiliaireL;
	nombreW += nombreAuxiliaireW;
	nombreC += nombreAuxiliaireC;
	// affichage des compteurs propres au fichier courant :
	wcAffichage(nombreAuxiliaireL,nombreAuxiliaireW,nombreAuxiliaireC,nomDeFichier);
    }
    
    protected static void wcParFichier (final String nomDeFichier) throws IOException
    {
	FileReader fr = null;
	try {
	    // le flux d'entre (qu'il soit de type flux sur un
	    // fichier ou flux sur l'entre standard) est lu avec un
	    // tampon d'entre dans un soucis d'efficacit :
	    if ("-".equals(nomDeFichier))
		wcAuxiliaire(new BufferedReader(new InputStreamReader(System.in)),null);
	    else {
		fr = new FileReader(nomDeFichier);
		wcAuxiliaire(new BufferedReader(fr),nomDeFichier);
		if (fr != null) fr.close();
	    }
	}
	catch (FileNotFoundException uneException) {
	    System.err.println("Problme : " + uneException);
	}
	catch (IOException uneException) {
	    System.err.println("Problme : " + uneException);
	    if (fr != null) fr.close();
	}
    }

    public static void analyseOptions (String lesOptions)
    {
	compterL = false;
	compterW = false;
	compterC = false;
	if (lesOptions.indexOf('l') > -1) compterL=true;
	if (lesOptions.indexOf('w') > -1) compterW=true;
	if (lesOptions.indexOf('c') > -1) compterC=true;	
    }
    
    public static void main (String[] arguments) throws IOException
    {
	int numroDuPremierFichier = 0;
	
	if (arguments.length == 0)
	    usage();
	else {
	    if ((arguments[0].length() > 1) && arguments[0].startsWith("-")) {
		analyseOptions(arguments[0]);
		numroDuPremierFichier = 1;
	    }
	    for (int i=numroDuPremierFichier ; i<arguments.length ; i++)
		wcParFichier(arguments[i]);
	    if (arguments.length > 1+numroDuPremierFichier)
		wcAffichage(nombreL,nombreW,nombreC,null);
	}
    }
}
