import java.util.Properties;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import com.sun.image.codec.jpeg.*;
import java.io.*;

class MandelbrotParallele
{
    private double xMin,xMax,yMin,yMax,xPas,yPas;
    private int xResolution,yResolution,profondeur;
    private String nomFichierImage;
    private Properties valeursInitiales;
    private BufferedImage monImage;
    private WritableRaster tamponImage;
    
    MandelbrotParallele(String nomDeFichierDInitialisation)
    {
	// cette mthode reste inchange par rapport  la version squentielle :
	valeursInitiales=new Properties();
	try {
            InputStream entre=
		new FileInputStream(new File(nomDeFichierDInitialisation));
            valeursInitiales.load(entre);
            // System.err.println("Chargement du fichier d'initialisation termin.");
	} catch (IOException e) {
            System.err.println("Pas de fichier d'initialisation.");
        }
	xMin = affecterDouble("xMin","-2.0");
	xMax = affecterDouble("xMax","0.5");
	yMin = affecterDouble("yMin","-1.25");
	yMax = affecterDouble("yMax","1.25");
	xResolution = affecterInt("xResolution","300");
	yResolution = affecterInt("yResolution","300");
	profondeur = affecterInt("profondeur","55");
	nomFichierImage = affecterString("nomDuFichierImage","Mandelbrot.jpg");
	xPas = (xMax - xMin) / xResolution;
	yPas = (yMax - yMin) / yResolution;
	monImage = new BufferedImage(xResolution,yResolution,
				     BufferedImage.TYPE_INT_RGB);
        tamponImage = monImage.getRaster();
    }

    String affecterString(String clef,String valeurParDfaut)
    {
	// cette mthode reste inchange par rapport  la version squentielle :
	String tmp = valeursInitiales.getProperty(clef,valeurParDfaut);
	valeursInitiales.setProperty(clef,tmp);
	return tmp;
    }

    int affecterInt(String clef,String valeurParDfaut)
    {
	// cette mthode reste inchange par rapport  la version squentielle :
	int tmp;
	try {
	    tmp = Integer.parseInt(valeursInitiales.
				   getProperty(clef,valeurParDfaut));
	} catch (NumberFormatException e) {
	    System.err.println("La valeur initiale pour l'attribut " 
			       + clef + " doit tre de type entier !");
	    tmp = Integer.parseInt(valeurParDfaut);
	}
	valeursInitiales.setProperty(clef,Integer.toString(tmp));
	return tmp;
    }

    double affecterDouble(String clef,String valeurParDfaut)
    {
	// cette mthode reste inchange par rapport  la version squentielle :
	double tmp;
	try {
	    tmp = Double.parseDouble(valeursInitiales.
				     getProperty(clef,valeurParDfaut));
	} catch (NumberFormatException e) {
	    System.err.println("La valeur initiale pour l'attribut " 
			       + clef + " doit tre de type flottant !");
	    tmp = Double.parseDouble(valeurParDfaut);
	}	
	valeursInitiales.setProperty(clef,Double.toString(tmp));
	return tmp;
    }

    void calculer(int nbThreads)
    {
	Thread[] bandes = new Thread[nbThreads];
	double yMinBande = yMin;
	int i,jMinBande = 0,
	    yResolutionBande = (int)Math.floor((double)yResolution/(double)nbThreads);

        for(i=0 ; i < nbThreads ; i++) {
	    // instanciation des threads :
	    bandes[i] = new Thread(
		new MandelbrotParBande(xMin,yMinBande,
				       xPas,yPas,jMinBande,
				       xResolution,yResolutionBande,
				       profondeur,tamponImage));
	    // lancement des threads :
            bandes[i].start();
	    // mise  jour des compteurs :
	    jMinBande += yResolutionBande;
	    yMinBande += yResolutionBande*yPas;
        }
	// barrire de synchronisation gnrale :
        for(i=0 ; i < nbThreads ; i++) {
            try {
                bandes[i].join();
            }
            catch (InterruptedException e){
            }
        }
    }
    
    void sauverImageRsultat() throws java.io.IOException
    {
	// cette mthode reste inchange par rapport  la version squentielle :
	FileOutputStream sortie=new FileOutputStream(new File(nomFichierImage));
        JPEGImageEncoder monEncodeur = JPEGCodec.createJPEGEncoder(sortie);
        JPEGEncodeParam param = monEncodeur.getDefaultJPEGEncodeParam(monImage);
        param.setQuality(1.0f, false);
        monEncodeur.setJPEGEncodeParam(param);
	
        monEncodeur.encode(monImage);
	sortie.close();
    }

    void enregistrerLesProprits(String nomDeFichierDInitialisation)
    {
	// cette mthode reste inchange par rapport  la version squentielle :
	try {
            OutputStream sortie =
		new FileOutputStream(new File(nomDeFichierDInitialisation));
            valeursInitiales.
		store(sortie,"Valeurs initiales - ensemble de Mandelbrot");
        }
        catch (IOException e) {
            System.err.println("Problme  l'enregistrement des valeurs initiales...");
        }
    }
    
    public static void main (String[] arguments) throws java.io.IOException
    {
        int nbThreads = Integer.parseInt(arguments[0]);

	long dateDbut = System.currentTimeMillis(); // mesure du temps
  	MandelbrotParallele bdpd = new MandelbrotParallele("Mandelbrot.ini");
	bdpd.calculer(nbThreads);
	bdpd.sauverImageRsultat();
	long dateFin = System.currentTimeMillis(); // mesure du temps
	bdpd.enregistrerLesProprits("Mandelbrot.ini");
	System.err.println(nbThreads + " proc : temps d'excution = " +
			   (dateFin-dateDbut) + " ms");
    }
}
