import java.util.Properties;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import com.sun.image.codec.jpeg.*;
import java.io.*;

class MandelbrotSequentiel
{
    /* 
     * Determiner l'ensemble des points C du plan complexe tels que la suite
     * complexe des (Z(n)) converge, avec :
     *        Z(0) = 0 
     *        Z(n+1) = Z(n)*Z(n) + C 
     */
    private double xMin,xMax,yMin,yMax,xPas,yPas;
    private int xResolution,yResolution,profondeur;
    private String nomFichierImage;
    private Properties valeursInitiales;
    private BufferedImage monImage;
    private WritableRaster tamponImage;
    
    MandelbrotSequentiel(String nomDeFichierDInitialisation)
    {
	valeursInitiales=new Properties();
	try {
            InputStream entre=
		new FileInputStream(new File(nomDeFichierDInitialisation));
            // chargement de la table des valeurs initiales depuis un fichier
            // du disque :
            valeursInitiales.load(entre);
            System.err.println("Chargement du fichier d'initialisation termin.");
	} catch (IOException e) {
            System.err.println("Pas de fichier d'initialisation.");
        }
	xMin = affecterDouble("xMin","-2.0");
	xMax = affecterDouble("xMax","0.5");
	yMin = affecterDouble("yMin","-1.25");
	yMax = affecterDouble("yMax","1.25");
	xResolution = affecterInt("xResolution","300");
	yResolution = affecterInt("yResolution","300");
	profondeur = affecterInt("profondeur","55");
	nomFichierImage = affecterString("nomDuFichierImage","Mandelbrot.jpg");
	// vrification du bon chargement des valeurs initiales :
	valeursInitiales.list(System.err);
	// affectation des dernires valeurs initiales :
	xPas = (xMax - xMin) / xResolution;
	yPas = (yMax - yMin) / yResolution;
	monImage = new BufferedImage(xResolution,yResolution,
				     BufferedImage.TYPE_INT_RGB);
        tamponImage = monImage.getRaster();
    }

    String affecterString(String clef,String valeurParDfaut)
    {
	String tmp = valeursInitiales.getProperty(clef,valeurParDfaut);
	valeursInitiales.setProperty(clef,tmp);
	return tmp;
    }

    int affecterInt(String clef,String valeurParDfaut)
    {
	int tmp;
	try {
	    tmp = Integer.parseInt(valeursInitiales.
				   getProperty(clef,valeurParDfaut));
	} catch (NumberFormatException e) {
	    System.err.println("La valeur initiale pour l'attribut " 
			       + clef + " doit tre de type entier !");
	    tmp = Integer.parseInt(valeurParDfaut);
	}
	valeursInitiales.setProperty(clef,Integer.toString(tmp));
	return tmp;
    }

    double affecterDouble(String clef,String valeurParDfaut)
    {
	double tmp;
	try {
	    tmp = Double.parseDouble(valeursInitiales.
				     getProperty(clef,valeurParDfaut));
	} catch (NumberFormatException e) {
	    System.err.println("La valeur initiale pour l'attribut " 
			       + clef + " doit tre de type flottant !");
	    tmp = Double.parseDouble(valeurParDfaut);
	}	
	valeursInitiales.setProperty(clef,Double.toString(tmp));
	return tmp;
    }

    void calculer()
    {
	// La primitive de calcul de la fractale  proprement parler :
	double rC,iC;         // Parties reelles et imaginaires de C
	double rZ,iZ;         // Parties reelles et imaginaires de Z
	double rZ2,iZ2,rZxiZ;
	int iter,i,j;
  	double couleur;
	
	for(j=0 , iC=yMin ; j<yResolution ; j++ , iC+=yPas)
	    for (i=0 , rC=xMin ; i<xResolution ; i++ , rC+=xPas) {
		// pour chaque point de la portion de plan complexe d'affixe
		// C=rC+i*iC, on calcule la suite des Zn tels que Z(n+1) =
		// Z(n)*Z(n)+C et Z(0)=0. En pratique, aprs initialisation de
		// Z  0, on itre la boucle suivante un nombre de fois au
		// plus gal  la valeur du paramtre "profondeur" (la boucle
		// s'arrte entre-temps si la valeur du module de Z dpasse 2) :
		rZ = 0.0;
		iZ = 0.0;
		iter = 0;
		do {
		    rZ2 = rZ * rZ;
		    iZ2 = iZ * iZ;
		    rZxiZ = rZ * iZ;        
		    rZ = rZ2 - iZ2 + rC;
		    iZ = 2 * rZxiZ + iC;
		    iter++;
		} while ((iter < profondeur) && ((rZ2 + iZ2) <= 4));
		// la couleur du pixel correspondant est obtenue par rapport
		// du nombre d'itrations effectifs avant "divergence" sur le
		// nombre total fix initialement d'itrations. Les
		// composantes rouge, verte et bleue ont toutes la mme valeur
		// puisque l'on cherche  gnrer une image dans les dgrads
		// de gris.
		couleur = (iter*255)/profondeur;
		tamponImage.setPixel(i,j,new double[]{couleur,couleur,couleur});
	    }
    }
    
    void sauverImageRsultat() throws java.io.IOException
    {
	// cette primitive permet de sauver le tableau de pixels "tamponImage"
	// sur un fichier du disque aprs encodage au format JPEG :
	FileOutputStream sortie=new FileOutputStream(new File(nomFichierImage));
        JPEGImageEncoder monEncodeur = JPEGCodec.createJPEGEncoder(sortie);
        JPEGEncodeParam param = monEncodeur.getDefaultJPEGEncodeParam(monImage);
	// l'encodage JPEG se fait sans perte de qualit avec un faible taux
	// de compression (valeur 1.0f passe en premier argument dans la
	// mthode setQuality) :
        param.setQuality(1.0f, false);
        monEncodeur.setJPEGEncodeParam(param);
	
        monEncodeur.encode(monImage);
	sortie.close();
    }

    void enregistrerLesProprits(String nomDeFichierDInitialisation)
    {
	try {
            OutputStream sortie =
		new FileOutputStream(new File(nomDeFichierDInitialisation));
            // enregistrement de la table des valeurs initiales dans un
            // fichier sur le disque :
            valeursInitiales.
		store(sortie,"Valeurs initiales - ensemble de Mandelbrot");
            System.err.println("Fin de l'enregistrement des valeurs initiales.");
        }
        catch (IOException e) {
            System.err.println("Problme  l'enregistrement des valeurs initiales...");
        }
    }
    
    public static void main (String[] arguments) throws java.io.IOException
    {
	long dateDbut = System.currentTimeMillis(); // mesure du temps
  	MandelbrotSequentiel bdpd = new MandelbrotSequentiel("Mandelbrot.ini");
	bdpd.calculer();
	bdpd.sauverImageRsultat();
	long dateFin = System.currentTimeMillis(); // mesure du temps
	bdpd.enregistrerLesProprits("Mandelbrot.ini");
	System.err.println("Temps d'excution = " + (dateFin-dateDbut) + " ms");
    }
}
