import java.net.*;
import java.io.*;
import java.util.*;

class ServeurDeFichierAuxiliaire implements Runnable
{
    // chaque instance de cette classe auxiliaire est charge de la gestion
    // d'une connexion prcise sur un serveur de fichier prcis (c'est--dire
    // sur une instance de la classe ServeurDeFichier) :
    private Socket laConnexion;
    private boolean continuer = true;
    private BufferedReader entre;
    private PrintWriter sortie;

    ServeurDeFichierAuxiliaire(Socket laConnexion)
    {
	this.laConnexion = laConnexion;
	try {
	    entre = 
		new BufferedReader(new InputStreamReader(laConnexion.getInputStream()));
	    sortie = new PrintWriter(laConnexion.getOutputStream(),true);
	} catch (IOException uneException) {
	    System.err.println(uneException);	    
	}
    }

    String traiterRequte() throws IOException
    {
	String ligne,fichier=null;
	int position=0;
	
	while ((ligne = entre.readLine()) != null)
	    if (ligne.startsWith("GET")) {
		try {
		    position = ligne.indexOf("/");
		    fichier = ligne.substring(position,ligne.indexOf(" ",position));
		}
		catch (IndexOutOfBoundsException uneException) {
		    fichier = ligne.substring(position);
		}
		finally {
		    break;
		}
	    }
	return fichier;
    }
    
    public void run()
    {
	try {
	    // lecture et traitement de la requte en provenance du client :
	    String nomDuFichier = traiterRequte();
	    if (nomDuFichier != null) {
		// extraction des donnes du fichier ayant pour nom
		// "nomDuFichier" et placement de celles-ci dans le tableau de
		// caractres "contenu" :
		File leFichier = new File(".",nomDuFichier);
		int taille = (int) leFichier.length();
		char[] contenu = new char[taille];
		BufferedReader tamponSurFichier = 
		    new BufferedReader(new FileReader(leFichier));
		tamponSurFichier.read(contenu,0,contenu.length);
		tamponSurFichier.close();
		// criture du tableau "contenu" sur le flux en criture
		// associ  la Socket courante :
		sortie.print(enTte(nomDuFichier,taille));
		sortie.print(contenu);
		sortie.flush();
		// fermeture des flux lis  la socket :
 		sortie.close();
 		entre.close();
	    }
	    // fermeture de la connexion :
 	    laConnexion.close();
	}
	catch (FileNotFoundException uneException) {
	    sortie.print(fichierInconnu());
	    sortie.flush();
	    sortie.close();
	}
	catch (IOException uneException) {
	    System.err.println(uneException);
	}
    }
    
    boolean continuer()
    {
	return continuer;
    }

    char[] enTte(String nomDuFichier,int taille)
    {
	// cette mthode gnre, sous forme d'un tableau de caractres, une
	// en-tte de rponse HTTP valide, charge de signifier au client HTTP
	// que sa requte a t traite avec succs. Elle affecte notamment
	// une valeur correcte au champ "Content-type" en fonction de
	// l'extension du fichier  envoyer :
	String type = (nomDuFichier.endsWith(".html")) ? "text/html" : "text/plain";
	String chane = new StringBuffer("HTTP/1.0 200 OK\r\nDate: ").
	    append(new Date().toString()).
	    append("\r\nServer: perfectible en tout point/0.0.0.1\r\nContent-type: ").
	    append(type).append("\r\nContent-length: ").append(taille).
	    append("\r\n\r\n").toString();
	return chane.toCharArray();
    }

    char[] fichierInconnu()
    {
	// cette mthode gnre, sous forme d'un tableau de caractres, une
	// en-tte de rponse HTTP valide, charge de signifier au client HTTP
	// que sa requte n'a pas pu tre satisfaite car la ressource demande
	// n'a pas t trouve :
	String chane = new StringBuffer("HTTP/1.0 404 Not Found\r\nDate: ").
	    append(new Date().toString()).
	    append("\r\nServer: perfectible en tout point/0.0.0.1\r\n").
	    append("Content-type: text/plain\r\n\r\n").
	    append("Document introuvable !!!\r\n").
	    toString();
	return chane.toCharArray();
    }
}
